package com.yeejoin.amos.safety.business.controller;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.typroject.tyboot.core.foundation.context.RequestContext;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yeejoin.amos.component.feign.config.InnerInvokException;
import com.yeejoin.amos.component.feign.model.FeignClientResult;
import com.yeejoin.amos.feign.privilege.Privilege;
import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;
import com.yeejoin.amos.safety.business.vo.CommonRequest;
import com.yeejoin.amos.safety.business.vo.DaoCriteria;
import com.yeejoin.amos.safety.business.vo.QueryOperatorEnum;
import com.yeejoin.amos.safety.business.vo.ReginParams;
import com.yeejoin.amos.safety.business.vo.StringUtil;

import springfox.documentation.annotations.ApiIgnore;

/**
 * <pre> 基础控制器 </pre>
 * @author  as-hanshipeng
 * @version  $Id: BaseController.java, v 0.1 2016-12-15 上午08:41:59 as-hanshipengExp $
 */
@ApiIgnore
@RestController
@RequestMapping("/base")
//@Authorization
public class BaseController {

	
	@Autowired
	protected HttpServletRequest request;
	
	
	 @Autowired
	    private RedisTemplate<String, String> redisTemplate;
	  

	protected String getToken() {
     String authToken = request.getHeader("token");
     if (authToken == null) {
         authToken = request.getHeader("X-Access-Token");
     }
     return authToken;
 }
 
 protected String getProduct() {
     String authToken = request.getHeader("product");
     if (authToken == null) {
         authToken = request.getParameter("product");
     }
     return authToken;
 }

 
 protected String getAppKey() {
     String authToken = request.getHeader("appKey");
     if (authToken == null) {
         authToken = request.getParameter("appKey");
     }
     return authToken;
 }
 
 //redi缓存选择的用户信息
 private String buildKey(String userId, String token) {
     //return "region_" + userId + "_" + token.substring(0, token.indexOf('_', 1));
     return "region_" + userId + "_" + token;

 }

 protected void saveSelectedOrgInfo(ReginParams reginParams) {
     redisTemplate.opsForValue().set(buildKey(getUserId(), getToken()), JSONObject.toJSONString(reginParams));
 }

 protected ReginParams getSelectedOrgInfo() {
     return JSON.parseObject(redisTemplate.opsForValue().get(buildKey(getUserId(), getToken())), ReginParams.class);
 }
	
	
	
	/**
	 * 当前登录用户信息
	 */
	 protected AgencyUserModel getUserInfo() {
	    	AgencyUserModel userModel=null;
	    	if(getToken()!=null){
	    	RequestContext.setToken(getToken());
	    	RequestContext.setProduct(getProduct());
	    	RequestContext.setAppKey(getAppKey());
	    	
	    	FeignClientResult feignClientResult;
			try {
				feignClientResult = Privilege.agencyUserClient.getme();
		    	 userModel = (AgencyUserModel) feignClientResult.getResult();

			} catch (InnerInvokException e) {
				e.printStackTrace();
			}
			
	    	}
	        return userModel;

	       // return CurrentAuthentication.getAuthenticatedUserThreadLocal();
	    }
	
	protected String getUserId() 
	{
		String userId = null;
		if (getUserInfo() != null)
		{
			userId = getUserInfo().getUserId();
		}
		return userId;
	}
	
	
	
	
	
	
	
	/**
	 * 获取最大数据位置（分页）
	 * 
	 * @param page
	 * @return
	 */
	public int getMaxResult(Pageable page) {
		if(page==null||page.getPageNumber()==-1){
			return Integer.MAX_VALUE;
		}
		return page.getPageSize();
	}

	/**
	 * 获取起始数据位置（分页）
	 * 
	 * @param page
	 * @return
	 */
	public int getFirstResult(Pageable page) {
		if(page==null||page.getPageNumber()==-1){
			return 0;
		}
		return page.getPageNumber() * page.getPageSize();
	}
		
	/**
	 * 获取当前用户所属公司名称
	 * @return
	 */
	 protected String getCurrentCompanyNameOfUser() {
			if (getUserInfo() == null) {
				return "";
			}
	return getSelectedOrgInfo().getCompany().getCompanyName();
		}
	/**
	 * 获取当前用户所属部门名称
	 * @return
	 */
	protected String getCurrentDepartmentNameOfUser(){
		return getSelectedOrgInfo().getDepartment().getDepartmentName();
	}
	
	protected String getCurrentDepartmentNameOfUsercode(){
		return getSelectedOrgInfo().getDepartment().getDeptOrgCode();
	}
	/**
	 * 
	 * <pre>
	 * 构建当前用户所属公司的code
	 * 例如：
	 * 属于大队A（填充中队A对应的CompanyRank中的compCode：总队id*支队id*大队id）
	 *
	 * </pre>
	 *
	 */
	protected String buildCurrentCompCodeOfUser()
	{
		
		ReginParams reginParams =getSelectedOrgInfo();
		 if (reginParams == null) {
	            return null;
	        }
	        if (reginParams.getCompany() != null) {
	            return reginParams.getCompany().getOrgCode();
	        }
	        return null;
	}
	/**
	 * 
	 * <pre>
	 * 根据code获取公司ID
	 * </pre>
	 *
	 * @param code
	 * @return
	 */
	protected String getCompIdByCode(String code)
	{
		if (code.contains("-")) //公司+部门的code
		{
			String[] codes =  code.split("-");
			String compCode = codes[0];
			String[] comps =  compCode.split("\\*");
			return comps[comps.length-1];
		}else  //公司的code
		{
			String[] comps =  code.split("\\*");
			return comps[comps.length-1];
		}
	}
	/**
	 * 
	 * <pre>
	 * 根据code获取部门ID
	 * </pre>
	 *
	 * @param code
	 * @return
	 */
	protected String getDepmIdByCode(String code)
	{
		if (code.contains("-"))
		{
			String[] codes =  code.split("-");
			String depmCode = codes[1];
			String[] comps =  depmCode.split("#");
			return comps[comps.length-1];
		}
		return null;
	}
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属公司（包含子公司）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
	public List<DaoCriteria> buildCompDaoCriteriaInChildren(List<DaoCriteria> daoCriterias) 
	{
		/**
		 * 当前用户所属公司
		 */
		String companyCode = buildCurrentCompCodeOfUser();		
		//添加所属公司的过滤条件
		DaoCriteria daoCriteria = new DaoCriteria();
		daoCriteria.setPropertyName("orgCode");
		daoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
		daoCriteria.setValue(companyCode);
		daoCriterias.add(daoCriteria);
		//添加子公司下的过滤条件
		DaoCriteria compDaoCriteria = new DaoCriteria();
		compDaoCriteria.setPropertyName("orgCode");
		compDaoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
		compDaoCriteria.setValue(companyCode+"*%");
		daoCriterias.add(compDaoCriteria);
		return daoCriterias;
	}
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属公司（不包含子公司）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
	public List<DaoCriteria> buildCompDaoCriteriaOutChildren(List<DaoCriteria> daoCriterias) 
	{
		/**
		 * 当前用户所属公司
		 */
		//Company company = getUserInfo().getCompany();
		String companyCode = buildCurrentCompCodeOfUser();
		//添加所属公司的过滤条件
		DaoCriteria daoCriteria = new DaoCriteria();
		daoCriteria.setPropertyName("orgCode");
		daoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
		daoCriteria.setValue(companyCode);
		daoCriterias.add(daoCriteria);
		return daoCriterias;
	}
	/**
	 * 
	 * <pre>
	 * 构建当前用户所属组织机构（公司+部门）的code
	 * 例如：
	 * 属于大队A（填充中队A对应的CompanyRank中的compCode：总队id*支队id*大队id）
	 * 属于大队A中的部门1（填充大队A对应的CompanyRank中的compCode-部门1对应的DepartmentRank的depmCode：总队id*支队id*大队id-父部门id#本部门id）
	 *
	 * </pre>
	 *
	 */
	protected String buildCurrentOrgCodeOfUser()
	{
		StringBuffer orgCode = new StringBuffer();
		/**
		 * 当前用户所属公司
		 */
		//Company company = getUserInfo().getCompany();
		String companyCode = buildCurrentCompCodeOfUser();
		orgCode.append(companyCode);
		/**
		 * 当前用户所属部门
		 */
//		Department department = getUserInfo().getDepartment();
//		//如果所属部门为null,则表示该用户为公司级用户
//		if (department != null)
//		{
		String  DepmCode=getCurrentDepartmentNameOfUsercode();
			orgCode.append("-");
			orgCode.append(DepmCode);
//		}
		return orgCode.toString();
	}
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属组织机构（包含子机构）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
	public List<DaoCriteria> buildOrgDaoCriteriaInChildren(List<DaoCriteria> daoCriterias) 
	{
		if (getUserInfo() == null)
		{
			return daoCriterias;
		}
		/**
		 * 当前用户所属公司
		 */
//		Company company = getUserInfo().getCompany();
//		String companyCode = company.getCompCode();
		String companyCode = buildCurrentCompCodeOfUser();
		//添加子公司下的过滤条件
		DaoCriteria childCompDaoCriteria = new DaoCriteria();
		childCompDaoCriteria.setPropertyName("orgCode");
		childCompDaoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
		childCompDaoCriteria.setValue(companyCode+"*%");
		daoCriterias.add(childCompDaoCriteria);
		/*
		 * 
		 * 当前用户所属部门
		 */
		//Department department = getUserInfo().getDepartment();
		String  depmCode=getCurrentDepartmentNameOfUsercode();
		if (depmCode != null)//部门用户，添加所属部门及子部门的过滤条件
		{
			//String depmCode = department.getDepmCode();
			//添加所属部门的过滤条件
			DaoCriteria depmDaoCriteria = new DaoCriteria();
			depmDaoCriteria.setPropertyName("orgCode");
			depmDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
			depmDaoCriteria.setValue(companyCode+"-"+ depmCode);
			daoCriterias.add(depmDaoCriteria);
			//添加子部门下的过滤条件
			DaoCriteria childDepmDaoCriteria = new DaoCriteria();
			childDepmDaoCriteria.setPropertyName("orgCode");
			childDepmDaoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
			childDepmDaoCriteria.setValue(companyCode+"-"+ depmCode+ "#%");
			daoCriterias.add(childDepmDaoCriteria);
		}else //公司级用户，添加所属公司、及公司下所有部门的查询条件
		{
			//添加所属公司的过滤条件
			DaoCriteria compDaoCriteria = new DaoCriteria();
			compDaoCriteria.setPropertyName("orgCode");
			compDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
			compDaoCriteria.setValue(companyCode);
			daoCriterias.add(compDaoCriteria);
			//添加所属公司下所有部门的过滤条件
			DaoCriteria depmDaoCriteria = new DaoCriteria();
			depmDaoCriteria.setPropertyName("orgCode");
			depmDaoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
			depmDaoCriteria.setValue(companyCode+"-%");
			daoCriterias.add(depmDaoCriteria);
		}
		return daoCriterias;
	}
	/**
	 * 
	 * <pre>
	 * 生成通过当前用户所属组织机构（不包含子机构）过滤的查询条件
	 * </pre>
	 *
	 * @param daoCriterias 查询条件集合
	 * @return
	 */
	public List<DaoCriteria> buildOrgDaoCriteriaOutChildren(List<DaoCriteria> daoCriterias) 
	{
		if (getUserInfo() == null)
		{
			return daoCriterias;
		}
		/**
		 * 当前用户所属公司
		 */
		
//		Company company = getUserInfo().getCompany();
//		String	companyCode = company.getCompCode();
		String companyCode = buildCurrentCompCodeOfUser();

		/**
		 * 当前用户所属部门
		 */
		//Department department = getUserInfo().getDepartment();
		String  depmCode=getCurrentDepartmentNameOfUsercode();

		if (depmCode != null)//部门用户，添加所属部门的过滤条件
		{
			//String depmCode = department.getDepmCode();
			//添加所属部门的过滤条件
			DaoCriteria depmDaoCriteria = new DaoCriteria();
			depmDaoCriteria.setPropertyName("orgCode");
			depmDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
			depmDaoCriteria.setValue(companyCode+"-"+ depmCode);
			daoCriterias.add(depmDaoCriteria);
		}else //公司级用户，添加所属公司、及公司下所有部门的查询条件
		{
			//添加所属公司的过滤条件
			DaoCriteria compDaoCriteria = new DaoCriteria();
			compDaoCriteria.setPropertyName("orgCode");
			compDaoCriteria.setOperator(QueryOperatorEnum.EQUAL.getName());
			compDaoCriteria.setValue(companyCode);
			daoCriterias.add(compDaoCriteria);
			//添加所属公司下所有部门的过滤条件
			DaoCriteria depmDaoCriteria = new DaoCriteria();
			depmDaoCriteria.setPropertyName("orgCode");
			depmDaoCriteria.setOperator(QueryOperatorEnum.LIKE.getName());
			depmDaoCriteria.setValue(companyCode+"-%");
			daoCriterias.add(depmDaoCriteria);
		}
		return daoCriterias;
	}
	/**
	 * 
	 * <pre>
	 * 生成查询条件
	 * </pre>
	 * 
	 * @param queryRequests
	 * @return
	 */
	public List<DaoCriteria> buildDaoCriterias(List<CommonRequest> queryRequests) {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
//		
//		if (queryRequests == null) {
//			return daoCriterias;
//		}
//		String orgCode = null;
		if(queryRequests != null && !queryRequests.isEmpty()){
			for (CommonRequest query : queryRequests) {
				DaoCriteria criteria = new DaoCriteria();
				if(StringUtil.isNotEmpty(query.getValue())){
//					if(query.getName().equals("orgCode")){
//						orgCode = query.getValue().toString();
//						continue;
//					}
					//要查询的属性
					criteria.setPropertyName(propertyNameChange(query.getName()));
//					String column = criteria.getPropertyName();
//					if (!(query.getValue() instanceof Collection<?>) 
//							&& column.substring(column.length()-2,column.length()).toUpperCase().equals("ID")) {
//						criteria.setValue(Long.valueOf(query.getValue().toString()));
//					} else {
					//要查询的属性的值
					criteria.setValue(query.getValue());
//					}
					
					if (query.getType() == null || query.getType().equals(""))
					{
						criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
					}else 
					{
						//查询操作LIKE、EQUAL等
						String operator = QueryOperatorEnum.getEnum(query.getType()).getName();
						if (operator==null || "".equals(operator)) {
							criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
						}else if(operator.equals(QueryOperatorEnum.LIKE.getName())){
							setLikeCriteria(criteria,query);
						}else if(operator.equals(QueryOperatorEnum.BIGGER_EQUAL.getName())){
							criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
						}else if(operator.equals(QueryOperatorEnum.LESS_EQUAL.getName())){
							criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
						}else {
							criteria.setOperator(QueryOperatorEnum.getEnum(operator)
									.getName());
						}
					}
					daoCriterias.add(criteria);
				}
			}	
		}
		
//		if(isDirector()){
//			daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias,orgCode);
//		}else{
//			daoCriterias = buildOrgDaoCriteriaOutChildren(daoCriterias);
//		}
		return daoCriterias;

	}
	
	
	
	
	/**
	 * 
	 * <pre>
	 * 生成查询条件
	 * </pre>
	 *
	 * @param queryRequest
	 * @return
	 */
	public List<DaoCriteria> buildDaoCriteria(CommonRequest queryRequest) {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		
		daoCriterias = buildOrgDaoCriteriaInChildren(daoCriterias);
		
		if (queryRequest == null) {
			return daoCriterias;
		}

		DaoCriteria criteria = new DaoCriteria();
		criteria.setPropertyName(propertyNameChange(queryRequest.getName()));
		criteria.setValue(queryRequest.getValue());
		String operator = QueryOperatorEnum.getEnum(queryRequest.getType()).getName();
		if (!StringUtil.isNotEmpty(operator)) {
			criteria.setOperator(QueryOperatorEnum.EQUAL.getName());
		}else if(operator.equals(QueryOperatorEnum.LIKE.getName())){
			setLikeCriteria(criteria,queryRequest);
		}else {
			criteria.setOperator(QueryOperatorEnum.getEnum(operator).getName());
		}
		daoCriterias.add(criteria);
		
		return daoCriterias;
	}
	
	/**
	 * 
	 * <pre>
	 * 生成查询条件
	 * </pre>
	 *
	 * @param queryRequest
	 * @return
	 */
	public List<DaoCriteria> buildDaoCriteria() {
		List<DaoCriteria> daoCriterias = new ArrayList<DaoCriteria>();
		return daoCriterias;
	}
	

	public String propertyNameChange(String name) {
		return name;
	}
	
	/**
	 * 对like查询语句的内容进行特殊字符转义
	 * @param criteria
	 * @param query
	 */
	public void setLikeCriteria(DaoCriteria criteria,CommonRequest query){
		String value = (String) query.getValue();
		if(value != null){
			value = value.replace("%","/%").replace("_","/_");
			criteria.setValue("%"+value+"%");
			criteria.setOperator(QueryOperatorEnum.LIKE.getName());
		}
	}

}
