package com.yeejoin.amos.safety.business.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.yeejoin.amos.safety.business.service.INodeLogViewService;
import com.yeejoin.amos.safety.business.vo.CommonPageable;
import com.yeejoin.amos.safety.business.vo.CommonRequest;
import com.yeejoin.amos.safety.business.vo.CommonResponse;
import com.yeejoin.amos.safety.business.vo.CommonResponseUtil;
import com.yeejoin.amos.safety.business.vo.CompanyBo;
import com.yeejoin.amos.safety.business.vo.DaoCriteria;
import com.yeejoin.amos.safety.business.vo.DepartmentBo;
import com.yeejoin.amos.safety.business.vo.QueryOperatorEnum;
import com.yeejoin.amos.safety.business.vo.ReginParams;
import com.yeejoin.amos.safety.common.cache.CurUserCompanyCache;
import com.yeejoin.amos.safety.common.entity.NodeToipLogView;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
/**
 * 
 * <pre>
 * 日志管理
 * </pre>
 *
 * @author amos
 * @version $Id: NodeLogViewController.java, v 0.1 2019年1月2日 下午3:37:47 amos Exp $
 */
@RestController
@RequestMapping("/bizLog")
@Api(value = "bizLog", tags = { "日志管理API" })
public class NodeLogViewController extends BaseController
{
	@Autowired
	private INodeLogViewService nodeLogViewService;

	@PostMapping(value = "/query",produces = "application/json;charset=UTF-8")
	@ApiOperation(value = "分页查询消息", notes = "分页查询消息")
	public CommonResponse getLogInfos(
			@ApiParam(value = "查询条件", required = false) @RequestBody(required = false) List<CommonRequest> queryRequests,
			@ApiParam(value = "分页参数", required = true) CommonPageable commonPageable)
	{
		List<DaoCriteria> daoCriterias = buildDaoCriterias(queryRequests);
		daoCriterias = buildViewDaoCriteria(daoCriterias);
//		User currUser = getUserInfo();
//		if (currUser != null)
//		{
//			Department department = currUser.getDepartment();
//			if (department != null)//部门用户
//			{
//				//通过当前登录用户orgCode过滤
//				daoCriterias = buildViewDaoCriteria(daoCriterias);
//			}else 
//			{
//				String compCode = CurUserCompanyCache.getInstance().getCurCompany(getUserId(), getToken());
//				if (compCode != null)
//				{
//					//查询本单位及子单位下所有部门
//					daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.EQUAL.getName(), compCode));
//					daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), compCode+"*%"));
//					daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), "%,"+compCode+"*%"));
//					daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), compCode+"-%"));
//					daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), "%,"+compCode+"-%"));
//				}else
//				{
//					//无orgCode条件，通过当前登录用户orgCode过滤
//					daoCriterias = buildViewDaoCriteria(daoCriterias);
//				}
//			}
//		}
		Sort sort = new Sort(Sort.Direction.DESC,"time");
		commonPageable.setSort(sort);
		Page<NodeToipLogView> toipLogs = nodeLogViewService.findAll(daoCriterias, commonPageable);
		return CommonResponseUtil.success(toipLogs);
	}
	/**
	 * 
	 * <pre>
	 * 构建3D视图当前用户orgCode过滤条件（适应于Node3DVo和Link3DVo中拼接的orgCode）
	 * </pre>
	 *
	 * @return
	 */
	private List<DaoCriteria> buildViewDaoCriteria(List<DaoCriteria> daoCriterias) 
	{
		ReginParams currUser = getSelectedOrgInfo();
		if (currUser != null)
		{
			CompanyBo company = currUser.getCompany();
			DepartmentBo department = currUser.getDepartment();
			Long compCode = company.getCompanyOrgCode();
			if (department == null)
			{
				//单位用户，查询本单位及子单位下所有部门
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.EQUAL.getName(), compCode));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), compCode+"*%"));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), "%,"+compCode+"*%"));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), compCode+"-%"));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), "%,"+compCode+"-%"));
			}else 
			{
				//部门用户,查询当前部门
				String orgCode = company.getCompanyOrgCode()+"-"+department.getDeptOrgCode();
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.EQUAL.getName(), orgCode));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), orgCode+",%"));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), "%,"+orgCode+",%"));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), "%,"+orgCode));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), compCode+"*%"));
				daoCriterias.add(new DaoCriteria("orgCode", QueryOperatorEnum.LIKE.getName(), "%,"+compCode+"*%"));
			}
		}
		return daoCriterias;
	}
}
