package com.yeejoin.amos.safety.business.controller;

import java.io.IOException;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.yeejoin.amos.feign.privilege.model.AgencyUserModel;
import com.yeejoin.amos.safety.business.service.INode3DVoService;
import com.yeejoin.amos.safety.business.vo.CommonResponse;
import com.yeejoin.amos.safety.business.vo.CommonResponseUtil;
import com.yeejoin.amos.safety.business.vo.JSONUtil;
import com.yeejoin.amos.safety.business.vo.ReginParams;
import com.yeejoin.amos.safety.business.websocket.View3DWebSocket;
import com.yeejoin.amos.safety.common.cache.CurUserCompanyCache;
import com.yeejoin.amos.safety.common.entity.Node3DVo;
import com.yeejoin.amos.safety.common.entity.RiskPoint;
import com.yeejoin.amos.safety.common.entity.dto.PatrolCheckDTO;
import com.yeejoin.amos.safety.common.entity.dto.PatrolTaskDTO;
import com.yeejoin.amos.safety.common.enums.CheckStateEnum;
import com.yeejoin.amos.safety.common.enums.View3DNodeType;
import com.yeejoin.amos.safety.common.view.View3DHandler;
import com.yeejoin.amos.safety.core.remote.service.RemoteSecurityService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

/**
 * 
 * <pre>
 * 巡检Controller
 * </pre>
 *
 * @author amos
 * @version $Id: PatrolController.java, v 0.1 2018年12月3日 下午5:12:27 amos Exp $
 */
@RestController
@RequestMapping(value = "/patrol")
@Api(value = "patrol", tags = { "巡检API" })
public class PatrolController extends BaseController
{
	
	@Autowired
	private INode3DVoService node3DVoService;
	
	@Autowired
	private RemoteSecurityService remoteSecurityServer;
	
	private static final Logger logger = Logger.getLogger(PatrolController.class);
	/**
	 * 
	 * <pre>
	 * 新增巡检记录信息
	 * </pre>
	 *
	 * @param check
	 * @return
	 */
	@PostMapping(value = "/check",produces = "application/json;charset=UTF-8")
	@ApiOperation(value = "新增巡检记录信息", notes = "新增巡检记录信息")
	//@Authorization(ingore = true)
	public CommonResponse addXJCheck(
			@ApiParam(value = "巡检记录信息", required = true) @RequestBody List<PatrolCheckDTO> patrolCheckDTOs)
	{
		if (patrolCheckDTOs == null)
		{
			return CommonResponseUtil.failure("参数为null");
		}
		for (PatrolCheckDTO patrolCheckDTO : patrolCheckDTOs)
		{
			logger.info("接收到巡检数据："+JSONUtil.toJson(patrolCheckDTO));
			RiskPoint riskPoint = businessAIEntityConverter(patrolCheckDTO);
			//获取巡检点对应的node
			List<Node3DVo> patrolNode3dVos = node3DVoService.findByTypeAndOid(
					View3DNodeType.RISK.getName(), String.valueOf(riskPoint.getPointID()));
			Node3DVo patrolNode3dVo = null;
			if (patrolNode3dVos != null && patrolNode3dVos.size() > 0)
			{
				//更新节点显示名称
				patrolNode3dVo = patrolNode3dVos.get(0);
				patrolNode3dVo.setDisplayName(riskPoint.getPointName());
				if (riskPoint.getStatus() != null && 
						!riskPoint.getStatus().equals(patrolNode3dVo.getNodeState()))
				{
					//只有不合格状态有任务
					if (!riskPoint.getStatus().equals(CheckStateEnum.UNQUALIFIED.getName()))
					{
						patrolNode3dVo.setTaskOid(null);
					}
					//更新数据库巡检点状态
					patrolNode3dVo.setNodeState(riskPoint.getStatus());
					node3DVoService.save(patrolNode3dVo);
				}
			}
			//获取巡检人对应的node,漏检状态不需要发送巡检人位置
			if (riskPoint.getStatus() != null && 
						!riskPoint.getStatus().equals(CheckStateEnum.MISSPATROL.getName()))
			{
				List<Node3DVo> perNode3dVos = node3DVoService.findByTypeAndOid(
						View3DNodeType.PERSON.getName(), String.valueOf(riskPoint.getUserID()));
				Node3DVo perNode3dVo = null;
				if (perNode3dVos == null || perNode3dVos.size() == 0)
				{
					//用户无对应的3D节点，则进行新增
					AgencyUserModel user= remoteSecurityServer.getUserById( getToken(), getProduct(), getAppKey(),String.valueOf(riskPoint.getUserID()));
					perNode3dVo = new Node3DVo();
					perNode3dVo.setDisplayName(user.getUserName());
					perNode3dVo.setType(View3DNodeType.PERSON.getName());
					perNode3dVo.setOid(String.valueOf(user.getUserId()));
				}else{
					perNode3dVo = perNode3dVos.get(0);
				}
				if (patrolNode3dVo != null && perNode3dVo != null)
				{
					try
					{
						//更新人员位置
						perNode3dVo.setPosition(patrolNode3dVo.getPosition());
						perNode3dVo.setRotation(patrolNode3dVo.getRotation());
						perNode3dVo.setScale(patrolNode3dVo.getScale());
						node3DVoService.save(perNode3dVo);
						//向前端发送巡检人位置消息
						View3DWebSocket.sendInfo(View3DHandler.getInstance().buildPersNodeWebsocket(perNode3dVo));
					}
					catch (IOException e)
					{
						logger.error("向前端发送巡检人位置消息失败："+e.getMessage());
						return CommonResponseUtil.failure("向前端发送巡检人位置消息失败："+e.getMessage());
					}
				}
			}
		}
		return CommonResponseUtil.success();
	}
	/**
	 * 
	 * <pre>
	 * 新增巡检任务信息
	 * </pre>
	 *
	 * @param patrolTaskDTOs
	 * @return
	 */
	@PostMapping(value = "/task",produces = "application/json;charset=UTF-8")
	@ApiOperation(value = "新增巡检任务信息", notes = "新增巡检任务信息")
	//@Authorization(ingore = true)
	public CommonResponse addXJTask(
			@ApiParam(value = "巡检记录信息", required = true) @RequestBody List<PatrolTaskDTO> patrolTaskDTOs)
	{
		if (patrolTaskDTOs == null)
		{
			return CommonResponseUtil.failure("参数为null");
		}
		for (PatrolTaskDTO patrolTaskDTO : patrolTaskDTOs)
		{
			logger.info("接收到巡检任务数据："+JSONUtil.toJson(patrolTaskDTO));
			RiskPoint riskPoint = businessAIEntityConverter(patrolTaskDTO);
			if (riskPoint.getPointID() != 0)//巡检点关联任务，需要发送巡检点状态消息
			{
				//获取巡检点对应的node
				List<Node3DVo> patrolNode3dVos = node3DVoService.findByTypeAndOid(
						View3DNodeType.RISK.getName(), String.valueOf(riskPoint.getPointID()));
				Node3DVo patrolNode3dVo = null;
				if (patrolNode3dVos != null && patrolNode3dVos.size() == 1)
				{
					patrolNode3dVo = patrolNode3dVos.get(0);
					//更新巡检点任务
					patrolNode3dVo.setTaskOid(String.valueOf(riskPoint.getTaskID()));
					node3DVoService.save(patrolNode3dVo);
				}
				
			}
		}
		return CommonResponseUtil.success();
	}
	/**
	 * 
	 * <pre>
	 * 保存当前登陆用户选择单位
	 * </pre>
	 *
	 * @param XJCompany
	 * @return
	 */
	@PostMapping(value = "/save/curCompany/{orgCode}",produces = "application/json;charset=UTF-8")
	@ApiOperation(value = "保存当前登陆用户选择单位", notes = "保存当前登陆用户选择单位")
	public CommonResponse saveCurrCompany(
			@ApiParam(value = "当前登陆用户所选单位信息", required = true) @RequestBody ReginParams reginParams)
	{
		//CurUserCompanyCache.getInstance().putCurCompanyMap(getUserId(), getToken(), orgCode);
		this.saveSelectedOrgInfo(reginParams);
        return CommonResponseUtil.success();
	}
	/**
	 * 
	 * <pre>
	 * 进行业务对象转换
	 * </pre>
	 *
	 * @param patrolCheckDTO
	 * @return
	 */
	private RiskPoint businessAIEntityConverter(PatrolCheckDTO patrolCheckDTO)
	{
		RiskPoint riskPoint = new RiskPoint();
		riskPoint.setAddress(patrolCheckDTO.getAddress() == null ? "" : patrolCheckDTO.getAddress());
		riskPoint.setPointID(patrolCheckDTO.getPointID());
		riskPoint.setPointNo(patrolCheckDTO.getPointNo() == null ? "" : patrolCheckDTO.getPointNo());
		riskPoint.setPointName(patrolCheckDTO.getPointName() == null ? "" : patrolCheckDTO.getPointName());
		riskPoint.setStatus(patrolCheckDTO.getStatus() == null ? "" : CheckStateEnum.getName(Integer.valueOf(patrolCheckDTO.getStatus())));
		riskPoint.setUserID(patrolCheckDTO.getUserID());
		riskPoint.setCheckID(patrolCheckDTO.getCheckID());
		riskPoint.setCheckTime(patrolCheckDTO.getCheckTime());
		riskPoint.setDepartment(patrolCheckDTO.getDepartment() == null ? "" : patrolCheckDTO.getDepartment());
		riskPoint.setPlanName(patrolCheckDTO.getPlanName() == null ? "" : patrolCheckDTO.getPlanName());
		riskPoint.setUserName(patrolCheckDTO.getUserName() == null ? "" : patrolCheckDTO.getUserName());
		riskPoint.setTelephone(patrolCheckDTO.getTelephone() == null ? "" : patrolCheckDTO.getTelephone());
		riskPoint.setCheckInput(patrolCheckDTO.getCheckInput());
		//测试
//		riskPoint.setPointID(68);
//		riskPoint.setPointNo("NO14");
//		riskPoint.setPointName("餐厅灶间一楼");
//		riskPoint.setStatus(patrolCheckDTO.getStatus() == null ? "" : CheckStateEnum.getName(Integer.valueOf(patrolCheckDTO.getStatus())));
//		riskPoint.setUserID(1);
//		riskPoint.setDepartment("工程部");
//		riskPoint.setUserName("张三");
		return riskPoint;
	}
	/**
	 * 
	 * <pre>
	 * 进行巡检任务业务对象转换
	 * </pre>
	 *
	 * @param patrolCheckDTO
	 * @return
	 */
	private RiskPoint businessAIEntityConverter(PatrolTaskDTO patrolTaskDTO)
	{
		RiskPoint riskPoint = new RiskPoint();
		riskPoint.setPointID(patrolTaskDTO.getPointID());
		riskPoint.setPointName(patrolTaskDTO.getPointName());
		riskPoint.setTaskID(patrolTaskDTO.getTaskID());
		riskPoint.setTaskName(patrolTaskDTO.getTaskName());
		riskPoint.setArranger(patrolTaskDTO.getArranger());
		riskPoint.setStatus(patrolTaskDTO.getStatus() == null ? "" : CheckStateEnum.getName(Integer.valueOf(patrolTaskDTO.getStatus())));
		riskPoint.setExecutor(patrolTaskDTO.getExecutor());
		riskPoint.setPlanTime(patrolTaskDTO.getPlanTime());
		riskPoint.setContent(patrolTaskDTO.getContent());
		riskPoint.setFishTime(patrolTaskDTO.getFishTime());
		
//		riskPoint.setPointID(68);
//		riskPoint.setPointName("餐厅灶间一楼");
//		riskPoint.setTaskID(25);
//		riskPoint.setTaskName("任务2");
//		riskPoint.setArranger("王五");
//		riskPoint.setExecutor("张三");
//		riskPoint.setUserID(1);
//		riskPoint.setPlanTime(new Date());
//		riskPoint.setContent("请关闭阀门");
		return riskPoint;
	}

}
