package com.yeejoin.amos.safety.business.websocket;

import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;

import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;
/**
 * 
 * <pre>
 * 3D视图WebSocket
 * </pre>
 *
 * @author amos
 * @version $Id: View3DSocket.java, v 0.1 2018年12月3日 下午6:14:02 amos Exp $
 */
@Component
@ServerEndpoint(value = "/view3d.ws")
public class View3DWebSocket
{
	private static final Logger log = Logger.getLogger(View3DWebSocket.class);

	// 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
	private static int onlineCount = 0;

	// concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
	private static CopyOnWriteArraySet<View3DWebSocket> webSocketSet = new CopyOnWriteArraySet<View3DWebSocket>();

	// 与某个客户端的连接会话，需要通过它来给客户端发送数据
	private Session session;

	/**
	 * 连接建立成功调用的方法
	 */
	@OnOpen
	public void onOpen(Session session)
	{
		this.session = session;
		webSocketSet.add(this); // 加入set中
		addOnlineCount(); // 在线数加1
		log.debug("有新连接加入！当前在线人数为" + getOnlineCount());
	}

	/**
	 * 连接关闭调用的方法
	 */
	@OnClose
	public void onClose()
	{
		webSocketSet.remove(this); // 从set中删除
		subOnlineCount(); // 在线数减1
		log.debug("有一连接关闭！当前在线人数为" + getOnlineCount());
	}

	/**
	 * 收到客户端消息后调用的方法
	 *
	 * @param message
	 *            客户端发送过来的消息
	 */
	@OnMessage
	public void onMessage(String message, Session session)
	{
		log.info("来自客户端的消息:" + message);
	}

	@OnError
	public void onError(Session session, Throwable error)
	{
		log.error("发生错误", error);
	}

	public void sendMessage(String message) throws IOException
	{
		this.session.getBasicRemote().sendText(message);
	}

	/**
	 * 群发自定义消息
	 */
	public static void sendInfo(String message) throws IOException
	{
		log.debug("——----View3DWebSocket开始群发消息------");
		log.debug("消息内容为：" + message);
		for (View3DWebSocket item : webSocketSet)
		{
			try
			{
				item.sendMessage(message);
			}
			catch (IOException e)
			{
				log.error(item.session.getId() + "消息发送失败", e);
				continue;
			}
		}
		log.debug("——----View3DWebSocket结束群发消息------");
	}

	public static synchronized int getOnlineCount()
	{
		return onlineCount;
	}

	public static synchronized void addOnlineCount()
	{
		View3DWebSocket.onlineCount++;
	}

	public static synchronized void subOnlineCount()
	{
		View3DWebSocket.onlineCount--;
	}
}
