package com.yeejoin.amos.safety.common.entity;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.annotations.Where;
import com.alibaba.fastjson.JSON;
import com.yeejoin.amos.safety.business.response.PointInfoResponse;
import com.yeejoin.amos.safety.common.entity.dto.CoordDTO;
import com.yeejoin.amos.safety.common.entity.dto.NodeConfigDTO;

/**
 * 
 * <pre>
 * 三维视图节点元素
 * </pre>
 *
 * @author amos
 * @version $Id: Node3DVo.java, v 0.1 2018年11月28日 下午6:23:35 amos Exp $
 */
@Entity
@Table(name = "toip_biz_node3dvo")
@Where(clause="is_delete=0")//表示未删除的数据
public class Node3DVo extends BusinessEntity
{
	/**
	 * <pre>
	 * 
	 * </pre>
	 */
	private static final long serialVersionUID = 8645988369602644725L;
	
	/**
	 * id
	 */
	@Id
    @GeneratedValue(generator = "uuid")
    @GenericGenerator(name = "uuid", strategy = "uuid2")
	@Column(name = "id", nullable = false, length = 36)
	protected String id;
	/**
	 * 显示名
	 */
	@Column(name = "display_name")
	private String displayName;
	/**
	 * 类型，nodeVo中填充View3DNodeType中的name值
	 */
	@Column(name = "type")
	private String type;
	/**
	 * 节点状态，nodeVo中填充View3DNodeState中的name值
	 */
	@Column(name = "node_state")
	private String nodeState;
	/**
	 * 所属对象Id
	 */
	@Column(name = "oid")
	private String oid;
	/**
	 * 任务所属对象Id
	 */
	@Column(name = "task_oid")
	private String taskOid;
	/**
	 * 显示图标
	 */
	@Column(name = "display_icon")
	private String displayIcon;
	
	/**
	 * 节点位置，包括x,y,z值
	 */
	@Column(name = "position")
	private String position;
	
	/**
	 * 节点位置，包括x,y,z值
	 */
	@Column(name = "rotation")
	private String rotation;
	
	/**
	 * 节点位置，包括x,y,z值
	 */
	@Column(name = "scale")
	private String scale;
	
	/**
	 * 节点配置信息
	 */
	@Column(name = "config")
	private String config;
	
	/**
	 * 点编号
	 */
	@Column(name = "dataCode")
	private String dataCode;
	
	/**
	 * 点等级
	 */
	@Column(name = "data_level")
	private String dataLevel;
	
	/**
	 * 保护对象名称
	 */
	@Column(name = "protect_obj_name")
	private String protectObjName;
	
	/**
	 * 线路名称
	 */
	@Column(name = "route_name")
	private String routeName;
	
	/**
	 * 节点配置信息
	 */
	@Transient
	private NodeConfigDTO nodeConfigDTO;
	
	/**
	 * 节点位置，包括x,y,z值
	 */
	@Transient
	private CoordDTO positionDTO;
	
	/**
	 * 节点位置，包括x,y,z值
	 */
	@Transient
	private CoordDTO rotationDTO;
	
	/**
	 * 节点位置，包括x,y,z值
	 */
	@Transient
	private CoordDTO scaleDTO;
	/**
	 * 组织机构（公司+部门），多个通过“，”进行隔开
	 */
	@Column(name = "org_code", length = 50)
	private String orgCode;
	
	/**
	 * 备注
	 */
	@Column(name = "remark")
	private String remark;

	@Column(name = "frequency")
	private Integer frequency;

	@Transient
	private PointInfoResponse pointInfoResponse;
	
	public String getId()
	{
		return id;
	}
	public void setId(String id)
	{
		this.id = id;
	}
	public String getDisplayName()
	{
		return displayName;
	}
	public void setDisplayName(String displayName)
	{
		this.displayName = displayName;
	}
	public String getType()
	{
		return type;
	}
	public void setType(String type)
	{
		this.type = type;
	}
	
	public String getNodeState()
	{
		return nodeState;
	}
	public void setNodeState(String nodeState)
	{
		this.nodeState = nodeState;
	}
	public String getOid()
	{
		return oid;
	}
	public void setOid(String oid)
	{
		this.oid = oid;
	}
	
	public String getTaskOid()
	{
		return taskOid;
	}
	public void setTaskOid(String taskOid)
	{
		this.taskOid = taskOid;
	}
	public String getDataCode() {
		return dataCode;
	}
	public void setDataCode(String dataCode) {
		this.dataCode = dataCode;
	}
	public String getDataLevel() {
		return dataLevel;
	}
	public void setDataLevel(String dataLevel) {
		this.dataLevel = dataLevel;
	}
	public String getProtectObjName() {
		return protectObjName;
	}
	public void setProtectObjName(String protectObjName) {
		this.protectObjName = protectObjName;
	}
	public String getRouteName() {
		return routeName;
	}
	public void setRouteName(String routeName) {
		this.routeName = routeName;
	}
	public void setNodeConfigDTO(NodeConfigDTO nodeConfigDTO) {
		this.nodeConfigDTO = nodeConfigDTO;
	}
	public String getDisplayIcon()
	{
		return displayIcon;
	}
	public void setDisplayIcon(String displayIcon)
	{
		this.displayIcon = displayIcon;
	}
	public String getPosition()
	{
		return position;
	}
	public void setPosition(String position)
	{
		this.position = position;
	}
	public String getRotation()
	{
		return rotation;
	}
	public void setRotation(String rotation)
	{
		this.rotation = rotation;
	}
	public String getScale()
	{
		return scale;
	}
	public void setScale(String scale)
	{
		this.scale = scale;
	}
	
	public String getOrgCode()
	{
		return orgCode;
	}
	public void setOrgCode(String orgCode)
	{
		this.orgCode = orgCode;
	}
	public String getRemark()
	{
		return remark;
	}
	public void setRemark(String remark)
	{
		this.remark = remark;
	}
	
	public String getConfig()
	{
		return config;
	}
	public void setConfig(String config)
	{
		this.config = config;
	}
	public NodeConfigDTO getNodeConfigDTO()
	{
		return config == null?null:JSON.parseObject(config, NodeConfigDTO.class);
	}
	public CoordDTO getPositionDTO()
	{
		return position == null?null:JSON.parseObject(position, CoordDTO.class);
	}
	public void setPositionDTO(CoordDTO positionDTO)
	{
		this.positionDTO = positionDTO;
	}
	public CoordDTO getRotationDTO()
	{
		return rotation == null?null:JSON.parseObject(rotation, CoordDTO.class);
	}
	public void setRotationDTO(CoordDTO rotationDTO)
	{
		this.rotationDTO = rotationDTO;
	}
	public CoordDTO getScaleDTO()
	{
		return scale == null?null:JSON.parseObject(scale, CoordDTO.class);
	}
	public void setScaleDTO(CoordDTO scaleDTO)
	{
		this.scaleDTO = scaleDTO;
	}
	public PointInfoResponse getPointInfoResponse()
	{
		return remark == null?null:JSON.parseObject(remark, PointInfoResponse.class);
	}
	public void setPointInfoResponse(PointInfoResponse pointInfoResponse)
	{
		this.pointInfoResponse = pointInfoResponse;
	}

	public Integer getFrequency() {
		return frequency;
	}

	public void setFrequency(Integer frequency) {
		this.frequency = frequency;
	}
}
 