package com.yeejoin.amos.safety.common.view;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import com.yeejoin.amos.safety.business.response.PatrolUserRespone;
import com.yeejoin.amos.safety.business.response.PointInfoResponse;
import com.yeejoin.amos.safety.business.response.RouteResponse;
import com.yeejoin.amos.safety.business.vo.DateUtil;
import com.yeejoin.amos.safety.business.vo.JSONUtil;
import com.yeejoin.amos.safety.business.vo.StringUtil;
import com.yeejoin.amos.safety.common.cache.UserPostionCache;
import com.yeejoin.amos.safety.common.entity.Link3DVo;
import com.yeejoin.amos.safety.common.entity.Node3DVo;
import com.yeejoin.amos.safety.common.entity.PatrolInfo;
import com.yeejoin.amos.safety.common.entity.dto.CoordDTO;
import com.yeejoin.amos.safety.common.entity.dto.LinkPathDTO;
import com.yeejoin.amos.safety.common.entity.dto.NodeConfigDTO;
import com.yeejoin.amos.safety.common.entity.dto.NodeCoordDTO;
import com.yeejoin.amos.safety.common.enums.CheckStateEnum;
import com.yeejoin.amos.safety.common.enums.View3DNodeType;

/**
 * 
 * <pre>
 * 3D视图处理类
 * </pre>
 *
 * @author amos
 * @version $Id: View3DHandler.java, v 0.1 2018年12月3日 上午11:51:12 amos Exp $
 */
public class View3DHandler
{
	/**
	 * 资源上下文实例
	 */
	private static View3DHandler instance;

	public static View3DHandler getInstance()
	{
		if (instance == null)
		{
			synchronized (View3DHandler.class)
			{
				if (instance == null)
				{
					instance = new View3DHandler();
				}
			}
		}
		return instance;
	}
	/**
	 * 
	 * <pre>
	 * 构建前端Json格式
	 * </pre>
	 *
	 * @param nodeVos
	 * @return
	 */
	public Map<String, List<Map<String, Object>>> buildViewNodeJson(List<Node3DVo> nodeVos)
	{
		//所有节点集合
		Map<String, List<Map<String, Object>>> viewTypeMap = new HashMap<String, List<Map<String, Object>>>();
		for (Node3DVo node3DVo : nodeVos)
		{
			if (node3DVo.getPositionDTO() == null || node3DVo.getRotationDTO() == null 
					|| node3DVo.getScaleDTO() == null)//如果节点无坐标
			{
				continue;
			}
			//每种类型节点集合
			List<Map<String, Object>> nodeList = viewTypeMap.get(node3DVo.getType());
			if (nodeList == null)
			{
				nodeList = new ArrayList<Map<String, Object>>();
			}
			Map<String, Object> nodeMap = new HashMap<String, Object>();
			nodeMap.put("id", node3DVo.getOid());
			
			nodeMap.put("type", node3DVo.getType());
			nodeMap.put("key", node3DVo.getId());
			PointInfoResponse pointInfoResponse = node3DVo.getPointInfoResponse();
			nodeMap.put("level", node3DVo.getDataLevel()!=null?node3DVo.getDataLevel():0);
			if (pointInfoResponse != null && pointInfoResponse.getPointLevel() != null)
			{
				nodeMap.put("level", pointInfoResponse.getPointLevel());
			}
			nodeMap.put("levelStr", "level_"+nodeMap.get("level").toString());//前端用于过滤几倍级别
			

			if (node3DVo.getType().equals(View3DNodeType.RISK.getName()))
			{
				if (node3DVo.getTaskOid() == null)
				{
					String[] title = {CheckStateEnum.getEnum(node3DVo.getNodeState()).getShowInfo()};
					nodeMap.put("title", title);
				}else 
				{
					String[] title = {CheckStateEnum.getEnum(node3DVo.getNodeState()).getShowInfo(), "task"};
					nodeMap.put("title", title);
				}
				int le =  CheckStateEnum.getEnum(node3DVo.getNodeState()).getValue();
				nodeMap.put("level",le);
				nodeMap.put("levelStr", "level_"+le);//前端用于过滤几倍级别
			}else 
			{
				nodeMap.put("title", node3DVo.getDisplayName());
			}
			if (StringUtil.isNotEmpty(node3DVo.getNodeState()) && "risk".equals(node3DVo.getType()))
			{
				
				int stateValue = CheckStateEnum.getEnum(node3DVo.getNodeState()).getValue();
				if (stateValue > CheckStateEnum.QUALIFIED.getValue())
				{
					nodeMap.put("showInfo", true);//显示title提示
					nodeMap.put("twinkle", true);//显示图标闪烁
				}else{
					nodeMap.put("showInfo", false);//隐藏title提示
					nodeMap.put("twinkle", false);//隐藏图标闪烁
				}
			} else if (StringUtil.isNotEmpty(node3DVo.getNodeState()) && "riskSource".equals(node3DVo.getType())) {
				
				if ("NORMAL".equals(node3DVo.getNodeState()))
				{
					nodeMap.put("showInfo", false);//隐藏title提示
					nodeMap.put("twinkle", false);//隐藏图标闪烁
				}else{
					nodeMap.put("showInfo", true);//显示title提示
					nodeMap.put("twinkle", true);//显示图标闪烁
					nodeMap.put("title", node3DVo.getNodeState());
				}
			} else{
				nodeMap.put("showInfo", false);//隐藏title提示
			}
			
			NodeConfigDTO nodeConfigDTO = node3DVo.getNodeConfigDTO();
			
			if (nodeConfigDTO == null){
				nodeConfigDTO = new NodeConfigDTO();
			}
			nodeMap.put("label", node3DVo.getDisplayName());
			nodeMap.put("displayIcon", node3DVo.getDisplayIcon());
			nodeMap.put("position", node3DVo.getPositionDTO());
			nodeMap.put("rotation", node3DVo.getRotationDTO());
			nodeMap.put("scale", node3DVo.getScaleDTO());
			nodeMap.put("objKey", nodeConfigDTO.getBelongObjModel());
			nodeMap.put("inDoor", nodeConfigDTO.getIsIndoor());
			nodeMap.put("frequency", node3DVo.getFrequency());
			nodeList.add(nodeMap);
			viewTypeMap.put(node3DVo.getType(), nodeList);
		}
		return viewTypeMap;
	}
	
	/**
	 * 
	 * <pre>
	 * 构建前端Json格式
	 * </pre>
	 *
	 * @param linkVos
	 * @return
	 */
	public List<Map<String, Object>> buildViewJson(List<Link3DVo> linkVos)
	{
		//构建链路返回数据
		List<Map<String, Object>> linkMaps = new ArrayList<Map<String, Object>>();
		for (Link3DVo link3DVo : linkVos)
		{
			LinkPathDTO linkPathDTO = link3DVo.getLinkPathDTO();
			if (linkPathDTO == null)//如果线路无path，则过滤掉
			{
				continue;
			}
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("id", link3DVo.getOid());
			map.put("name", link3DVo.getDisplayName());
			map.put("key", link3DVo.getId());
			LinkedList<NodeCoordDTO> nodeCoordDTOs = linkPathDTO.getNodeCoordDTOs();
			if (nodeCoordDTOs != null && nodeCoordDTOs.size() > 0)
			{
				CoordDTO[] coordDTOs = new CoordDTO[nodeCoordDTOs.size()];
				for (int i = 0; i < nodeCoordDTOs.size(); i++)
				{
					coordDTOs[i] = nodeCoordDTOs.get(i).getPositionDTO();
				}
				map.put("linkPath", coordDTOs);
				map.put("startPoint", coordDTOs[0]);
				map.put("endPoint", coordDTOs[coordDTOs.length - 1]);
			}
			linkMaps.add(map);
		}
		return linkMaps;
	}
	/**
	 * 
	 * <pre>
	 * 构建前端Json格式
	 * </pre>
	 *
	 * @param linkVos
	 * @return
	 */
	public List<Map<String, Object>> buildLinkSeachJson(List<Link3DVo> linkVos)
	{
		//构建链路返回数据
		List<Map<String, Object>> linkMaps = new ArrayList<Map<String, Object>>();
		for (Link3DVo link3DVo : linkVos)
		{
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("id", link3DVo.getOid());
			map.put("key", link3DVo.getId());
			map.put("name", link3DVo.getDisplayName());
			RouteResponse routeResponse = link3DVo.getRouteResponse();
			if (routeResponse != null)
			{
				map.put("leader", routeResponse.getBoss());
				map.put("telephone", routeResponse.getTel());
				StringBuilder stringBuilder = new StringBuilder();
				List<PatrolUserRespone> userRespones =  routeResponse.getPatrolUsers().stream().filter(user->user.getGroupName()!=null).collect(Collectors.toList());
				if (userRespones != null && userRespones.size() > 0)
				{
					for (int i = 0; i < userRespones.size(); i++)
					{
						if (i > 0)
						{
							stringBuilder.append("、");
						}
						stringBuilder.append(userRespones.get(i).getGroupName());
					}
				}
				map.put("region", stringBuilder.toString());
			}
			linkMaps.add(map);
		}
		return linkMaps;
	}
	/**
	 * 
	 * <pre>
	 * 构建巡检人位置消息json
	 * </pre>
	 *
	 * @param persNode3DVo
	 * @return
	 */
	public String buildPersNodeWebsocket(Node3DVo persNode3DVo)
	{
		Map<String, Object> nodeMap = new HashMap<String, Object>();
		nodeMap.put("id", persNode3DVo.getOid());
		StringBuilder stringBuilder = new StringBuilder();
		stringBuilder.append(persNode3DVo.getDisplayName()).append("(")
				.append(persNode3DVo.getOid()).append(")");
		nodeMap.put("title", stringBuilder.toString());
		nodeMap.put("label", persNode3DVo.getDisplayName());
		nodeMap.put("type", persNode3DVo.getType());
		nodeMap.put("topic", "base3d.view.point_" + persNode3DVo.getType());
		nodeMap.put("key", persNode3DVo.getId());
		nodeMap.put("level", 0);
		nodeMap.put("levelStr", "level_0");//前端用于过滤几倍级别
		nodeMap.put("position", persNode3DVo.getPositionDTO());
		nodeMap.put("rotation", persNode3DVo.getRotationDTO());
		nodeMap.put("scale", persNode3DVo.getScaleDTO());
		
		List<Number[]> pathArr = new ArrayList<Number[]>();
		// 上一个点 (用户id)
		CoordDTO prevCoord = UserPostionCache.getInstance().getPostionMap(persNode3DVo.getOid());
		if (prevCoord != null)
		{
			Number[] prevPoint = { prevCoord.getX(), prevCoord.getY(), prevCoord.getZ() };
			// 下一个点
			CoordDTO nextCoord = persNode3DVo.getPositionDTO();
			Number[] nextPoint = { nextCoord.getX(), nextCoord.getY(), nextCoord.getZ() };
			
			pathArr.add(prevPoint);
			pathArr.add(nextPoint);
			
			nodeMap.put("pathArr", pathArr);
		}
		nodeMap.put("showInfo", true);
		return JSONUtil.toJson(nodeMap);
	}
	
	private String getPatrolProperty(Object property){
		return property == null ? "" : property.toString();
	}
	
	public Map<String, Object> buildPatrolInfo(LinkedHashMap<String, Object> patrolInfo)
	{
		Map<String, Object> patrolMap = new HashMap<>();
		List<PatrolInfo> infoMap = new ArrayList<PatrolInfo>();
		
		PatrolInfo label = new PatrolInfo("label", "编号", getPatrolProperty(patrolInfo.get("pointNo")));
		PatrolInfo name = new PatrolInfo("name", "名称", getPatrolProperty(patrolInfo.get("pointName")));
		PatrolInfo status = new PatrolInfo("status", "状态", getPatrolProperty(patrolInfo.get("isOK")));
		PatrolInfo department = new PatrolInfo("department", "责任部门", getPatrolProperty(patrolInfo.get("group")));
		PatrolInfo chargeUser= new PatrolInfo("chargeUser", "责任人", getPatrolProperty(patrolInfo.get("checkUserName")));
		PatrolInfo contact= new PatrolInfo("contact", "联系方式", getPatrolProperty(patrolInfo.get("contact")));
		PatrolInfo link = new PatrolInfo("link", "巡检线路", getPatrolProperty(patrolInfo.get("routeName")));
		PatrolInfo time = new PatrolInfo("time", "巡检时间", patrolInfo.get("checkTime")==null?"":DateUtil.long2LongStr(Long.valueOf(patrolInfo.get("checkTime").toString())));
		PatrolInfo plan = new PatrolInfo("plan", "巡检计划", getPatrolProperty(patrolInfo.get("planRemark")));
		
		infoMap.add(label);
		infoMap.add(name);
		infoMap.add(status);
		infoMap.add(department);
		infoMap.add(chargeUser);
		infoMap.add(contact);
		infoMap.add(link);
		infoMap.add(time);
		infoMap.add(plan);
		
		patrolMap.put("info", infoMap);
		patrolMap.put("photosUrl", patrolInfo.get("photosUrl"));
		return patrolMap;
	}
	/**
	 * 
	 * <pre>
	 * 进行当前用户数据过滤
	 * </pre>
	 *
	 * @param node3DVos
	 * @param user 当前用户
	 * @return
	 */
	public List<Node3DVo> nodeDatafilter(List<Node3DVo> node3DVos)
	{
		/**
		 * 进行数据过滤
		 */
		List<Node3DVo> node3DVosfilter = new ArrayList<>();
		for (Node3DVo node3dVo : node3DVos)
		{
			if (StringUtil.isNotEmpty(node3dVo.getPositionDTO()) && StringUtil.isNotEmpty(node3dVo.getRotationDTO()) 
					&& StringUtil.isNotEmpty(node3dVo.getScaleDTO()))//如果节点无坐标，则过滤掉
			{
				node3DVosfilter.add(node3dVo);
			}
		}
		return node3DVosfilter;
	}
	/**
	 * 
	 * <pre>
	 * 进行线路数据过滤
	 * </pre>
	 *
	 * @param link3DVos
	 * @return
	 */
	public List<Link3DVo> linkDatafilter(List<Link3DVo> link3DVos)
	{
		/**
		 * 进行数据过滤
		 */
		List<Link3DVo> link3DVosfilter = new ArrayList<>();
		for (Link3DVo link3DVo : link3DVos)
		{
			if (StringUtil.isNotEmpty(link3DVo.getPath()))//如果线路无path，则过滤掉
			{
				link3DVosfilter.add(link3DVo);
			}
		}
		return link3DVosfilter;
	}

}